SpaceWire light driver
======================

The SpaceWire light Linux kernel module provides a seamless integration in the
networking subsystem for a SpaceWire light IP core with Avalon support. It
provides control and data transfer for the core and exposes a network device
(ethernet) forwarding Ethernet frames as SpaceWire packets. The IP core is
expected to be instantiated in the FPGA section of a Intel Cyclone V
System-On-Chip (SoC). It is based on the Altera Triple-Speed-Ethernet (TSE)
driver (drivers/net/ethernet/altera/) in the upstream Linux kernel tree.

 -  spw_main.c , spw_light.h
    were reworked from altera_tse_main.[ch] to support
    the custom SpaceWire light core

 -  crc16.c
    contains the CCSDS CRC16 calculation used for the CRC augmentation of the
    last byte of an incoming SpaceWire packet.

The following files were not modified from the upstream kernel version (v4.9).
'altera_msgdma.c' (see spw_light.patch) can be enhanced to support descriptor
identification.

 altera_msgdma.c
 altera_msgdma.h
 altera_msgdmahw.h
 altera_tse.h
 altera_utils.c
 altera_utils.h

Compiling
---------

make ARCH=arm CROSS_COMPILE=arm-linux-gnueabihf- -C [kernel source] \
	M=$PWD CONFIG_SPW_LIGHT=m


Module parameters
-----------------

 debug
	Message Level (-1: default, 0: no output, 16: all) (int)

 dma_rx_num
	Number of descriptors in the RX list
 dma_tx_num
	Number of descriptors in the TX list
 crc_fix
	Set missing byte augmentaion of packet CRC-16 


Required device node properties
-------------------------------

- compatible : Shall be "tug,spw-light-1.0"

- reg-names :
	"control_port" - SpaceWire core registers
	"rx_csr" - rx mSGDMA controller CSR register
	"rx_desc" - rx mSGDMA descriptor address
	"rx_resp" - rx mSGDMA response address
	"tx_csr" - tx mSGDMA CSR register
	"tx_desc" - tx mSGDMA descriptor address
- reg :
	Contains the addresses per reg-names.

- interrupt-names : Should contain:
	"rx_irq" - rx path DMA controller interrupt
	"tx_irq" - tx path DMA controller interrupt
	"control_irq" - SpaceWire core interrupt
- interrupts :
	Contains the interrupt-specifiers per interrupt-names.

- max-frame-size :
	Contains the maximum transmission unit (MTU).
	Absolute maximum value is the DMA buffer size.


Example:

spw_0: spw@1,00001000 {
	compatible = "tug,spw-light-1.0";

	reg-names = "control_port", "rx_csr", "rx_desc", "rx_resp", "tx_csr", "tx_desc";
	reg =  <1 0x00006200 0x00000020>,
	    <1 0x00006000 0x00000020>,
	    <1 0x00006020 0x00000020>,
	    <1 0x00006040 0x00000008>,
	    <1 0x00006100 0x00000020>,
	    <1 0x00006120 0x00000020>;

	interrupt-names = "rx_irq", "tx_irq", "control_irq";
	interrupts = <0 53 4>, <0 54 4>, <0 55 1>;

	max-frame-size = <2048>;
};


Open issues
-----------
 *  no handling for SpaceWire timecode
 *  missing ethtool support. This could be useful for e.g. link speed setting
 *  fix packet length issue
 *  proper (ethernet) frame type and handling
 *  dynamic setting of DMA buffer size


License
-------
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA,
or see <http://www.gnu.org/licenses/>.
